library(magrittr)
library(ready4)
ready4fun::write_fn_type_dirs()
# MANUAL STEP. Write all your functions to R files in the new "fns" directory.
fns_env_ls <- ready4fun::read_fns(c("data-raw/fns/","data-raw/mthds/"),
                                  fns_env = new.env(parent = globalenv()))
x_ready4fun_manifest <- ready4fun::make_pkg_desc_ls(pkg_title_1L_chr = "Youth Outcomes to Health Utility",
                                                    pkg_desc_1L_chr = "Tools for mapping measures routinely collected in youth mental health services to Quality Adjusted Life Years (QALYs). Part of the First Bounce model of primary youth mental health services.
  This development version of the youthu package has been made available as part of the process of testing and documenting the package. If you have any questions, please contact the authors (matthew.hamilton@orygen.org.au).
  The documentation for this package has been automatically generated by the ready4fun package and is therefore quite rudimentary. Further human authored documentation will follow in 2021.",
                                                    authors_prsn = c(utils::person(given = "Matthew",family = "Hamilton",email = "matthew.hamilton@orygen.org.au", role = c("aut", "cre"),comment = c(ORCID = "0000-0001-7407-9194")),
                                                                     utils::person(given = "Caroline",family = "Gao",email = "caroline.gao@orygen.org.au", role = c("aut"),comment = c(ORCID = "0000-0002-0987-2759")),
                                                                     utils::person("Orygen", role = c("cph", "fnd")),
                                                                     utils::person("Headspace", role = c( "fnd")),
                                                                     utils::person("National Health and Medical Research Council", role = c( "fnd"))),
                                                    urls_chr = c("https://ready4-dev.github.io/youthu/",
                                                                 "https://github.com/ready4-dev/youthu",
                                                                 "https://www.ready4-dev.com/")) %>%
  ready4fun::make_manifest(addl_pkgs_ls = ready4fun::make_addl_pkgs_ls(#depends_chr = c("eq5d","ggfortify"),
                                                                       suggests_chr = c("knitr","rmarkdown"),
                                                                       #imports_chr = c("betareg","knitrBootstrap")
                                                                       ),
                           build_ignore_ls = ready4fun::make_build_ignore_ls(file_nms_chr = c("initial_setup.R")),
                           check_type_1L_chr = "ready4",
                           copyright_holders_chr = "Orygen",
                           custom_dmt_ls = ready4fun::make_custom_dmt_ls(user_manual_fns_chr = c("add_utl_predn",
                                                                                                 "add_qalys_to_ds",
                                                                                                 "get_mdl_from_dv",
                                                                                                 "get_mdls_using_predrs",
                                                                                                 "get_tfmn_from_lup",
                                                                                                 "make_hlth_ec_smry",
                                                                                                 "make_sngl_grp_ds",
                                                                                                 "make_matched_ds",
                                                                                                 "rename_from_nmd_vec")),##
                           dev_pkgs_chr = c("ready4","ready4fun",
                                            "ready4use","ready4show",
                                            "youthvars", "TTU"),
                           lifecycle_stage_1L_chr = "experimental",
                           path_to_pkg_logo_1L_chr = "../../../../../Documentation/Images/youthu-logo/default.png",
                           pkg_dmt_dv_dss_chr = c("https://doi.org/10.7910/DVN/HLLXZN",
                                                  "https://doi.org/10.7910/DVN/2Y9VF9"),
                           ready4_type_1L_chr = "authoring")
mdl_ingredients_ls <- ready4fun::get_rds_from_dv("mdl_ingredients",
                                           dv_ds_nm_1L_chr = "https://doi.org/10.7910/DVN/DKDIB0")
  # mdls_smry_tb <-ready4use::ready4use_dataverses() %>%
  # tibble::add_case(data_repo_db_ui_chr = "https://doi.org/10.7910/DVN/DKDIB0", # NOT UP TO DATE
  #                  file_name_chr = "mdls_smry_tb",
  #                  file_type_chr = ".csv",
  #                  data_repo_file_ext_chr = ".tab") %>%
  # ready4::procure()
mdl_ingredients_ls$mdls_lup
# utils::data("predictors_lup", package = "youthvars")
datasets_ls <- list(
  mdl_ingredients_ls$mdls_lup %>%
  # tibble::tibble(mdl_nms_chr = mdls_smry_tb$Model %>% unique()) %>%
  #                     dplyr::mutate(predrs_ls = mdl_nms_chr %>% strsplit("_") %>% purrr::map(~ .x[.x %in% c(predictors_lup$short_name_chr)]),
  #                                   mdl_type_chr = mdl_nms_chr %>% strsplit("_") %>% purrr::map(~ .x[.x %in% c("GLM", "OLS")]) %>% purrr::flatten_chr(),
  #                                   tfmn_chr = mdl_nms_chr %>% purrr::map_chr(~stringr::str_sub(.x,start = 1 +  stringi::stri_locate_last(.x,fixed = "_")[1,1] %>% as.vector())))  %>%
                      ready4fun::make_pkg_ds_ls(db_1L_chr = "mdls_lup",
                                                  title_1L_chr = "Lookup table of prediction models",
                                                  desc_1L_chr = "A summary of the key descriptive features of the prediction models included in the youthu package."))
x_ready4use_manifest <- ready4use::make_pt_ready4use_manifest(x_ready4fun_manifest,
                                                              #constructor_r3 = x_ready4class_constructor,
                                                              pkg_ds_ls_ls = datasets_ls) %>%
  ready4use::ready4use_manifest() # Need to rebuild
x_xx <- ready4::author(x_ready4use_manifest)
# usethis::use_package("truncnorm")
# devtools::build_vignettes()
